# file: babylonianSqrt.py
# author: Bob Muller
#
# CS1101 Computer Science I
#
# The Babylonian bisection algorithm for approximating the sqrt(x).
# The determines a range lo..hi known to contain the sqrt. The midpoint
# is taken as a guess. If the guess is adequate, it is returned. If not,
# the algorithm repeats, this time having cut the search range in half.
#
# See:
# https://johncarlosbaez.wordpress.com/2011/12/02/babylon-and-the-square-root-of-2/
#
import random, math

acceptableError = 1e-8

# closeEnough : float * float -> bool
#
def closeEnough(a, b):
  return abs(a - b) <= acceptableError

# A helper function for tracing.
#
def show(i, lo, hi, guess, guessSquared, x):
  front = "try=" + str(i) + ", lo=" + str(lo) + ", hi=" + str(hi)
  back  = ", guess=" + str(guess) + ", error=" + str(abs(guessSquared - x))
  print front + back

# sqrt : float * int -> float
#
# Note the use of n as an optional argument with a default value of 100. Als
# note the return of None if an adequate approximation cannot be found.
#
def sqrt(x, n=100):
  def repeat(i, lo, hi):
    if i > n:
      print "sqrt: couldn't compute adequate approximation"
      return None
    else:
      guess = (hi + lo) / 2.0
      guessSquared = guess ** 2.0

      show(i, lo, hi, guess, guessSquared, x)
      
      if closeEnough(guessSquared, x):
        return guess
      else:
        if guessSquared > x:
          return repeat(i + 1, lo, guess)
        else:
          return repeat(i + 1, guess, hi)
  return repeat(1, 0, x)
  
